@@ -1,12 +1,10 @@
 # -*- coding: utf-8 -*-
 """Pyrgg functions module."""
-import datetime
+import os
+from random import randint
 from json import loads as json_loads
 from json import dump as json_dump
-import os
 from pickle import dump as pickle_dump
-from random import randint, uniform, choice
-from textwrap import fill
 from yaml import safe_dump as yaml_dump
 import pyrgg.params
 
@@ -30,6 +28,51 @@ def is_weighted(max_weight, min_weight, signed):
     return True
 
 
+def get_min_max_weight(weight_dic):
+    """
+    Get minimum and maximum weight values.
+
+    :param weight_dic: weight dictionary
+    :type weight_dic: dict
+    :return: minimum and maximum weight values
+    """
+    all_weights = [abs(w) for weights in weight_dic.values() for w in weights]
+    return min(all_weights), max(all_weights)
+
+
+def is_signed(weight_dic): # pragma: no cover
+    """
+    Check if the graph is signed.
+
+    :param weight_dic: weight dictionary
+    :type weight_dic: dict
+    :return: signed flag
+    """
+    return any([any([w < 0 for w in weights]) for weights in weight_dic.values()])
+
+
+def has_self_loop(edge_dic): # pragma: no cover
+    """
+    Check if the graph has self loops.
+    
+    :param edge_dic: edge dictionary
+    :type edge_dic: dict
+    :return: self looped flag
+    """
+    return any([v in edges for v, edges in edge_dic.items()])
+
+
+def is_multigraph(edge_dic):
+    """
+    Check if the graph is a multigraph.
+
+    :param edge_dic: edge dictionary
+    :type edge_dic: dict
+    :return: multigraph flag
+    """
+    return any([len(set(edges)) != len(edges) for edges in edge_dic.values()])
+
+
 def get_precision(input_number):
     """
     Return precision of input number.
@@ -40,65 +83,163 @@ def get_precision(input_number):
     """
     try:
         number_str = str(input_number)
-        _, decimalpart = number_str.split(".")
-        return len(decimalpart)
+        _, decimal_part = number_str.split(".")
+        return len(decimal_part)
     except Exception:
         return 0
 
 
+def threshold_calc(min_edge, max_edge, vertex_degree):
+    """
+    Calculate threshold for branch_gen_pyrgg function.
+
+    :param min_edge: minimum number of edges (connected to each vertex)
+    :type min_edge: int
+    :param max_edge: maximum number of edges (connected to each vertex)
+    :type max_edge: int
+    :param vertex_degree: vertex degree
+    :type vertex_degree: int
+    :return: threshold as int
+    """
+    threshold = min_edge
+    lower_limit = 0
+    upper_limit = max_edge - vertex_degree
+    if vertex_degree < min_edge:
+        lower_limit = min_edge - vertex_degree
+    if upper_limit > lower_limit:
+        threshold = randint(lower_limit, upper_limit)
+    return threshold
+
+
 def is_float(input_number):
     """
     Check input for float conversion.
 
     :param input_number: input number
-    :type input_number: float or int
+    :type input_number: float or int or str
     :return: result as bool
     """
     try:
-        _, decimalpart = divmod(float(input_number), 1)
+        _, decimal_part = divmod(float(input_number), 1)
     except TypeError:
         return False
     else:
-        return True if decimalpart else False
+        return True if decimal_part else False
 
 
-def convert_str_to_number(string):
+def handle_string(string):
+    """
+    Handle string and raise ValueError if it is empty.
+
+    :param string: input string
+    :type string: str
+    :return: result as str
+    """
+    if string == "":
+        raise ValueError
+    return string
+
+
+def handle_pos_int(input_number):
+    """
+    Handle input number and raise ValueError if it is negative.
+
+    :param input_number: input number
+    :type input_number: float or int or str
+    :return: result as int
+    """
+    val = int(input_number)
+    if val < 0:
+        raise ValueError
+    return val
+
+
+def handle_str_to_number(string):
     """
     Convert string to float or int.
 
     :param string: input string
     :type string: str
-    :return: float or int
+    :return: result as float or int
     """
     return float(string) if is_float(string) else int(string)
 
 
-def convert_str_to_bool(string):
+def handle_str_prob(string):
     """
-    Convert 0/1 string to bool.
+    Convert string to float and raise ValueError if string is invalid.
 
     :param string: input string
     :type string: str
-    :return: bool
+    :return: result as float
     """
-    return bool(int(string))
+    val = handle_str_to_number(string)
+    if val < 0:
+        raise ValueError
+    if val > 1:
+        raise ValueError
+    return val
 
 
-ITEM_CONVERTORS = {
-    "file_name": lambda x: x,
-    "output_format": int,
-    "weight": convert_str_to_bool,
-    "vertices": int,
-    "number_of_files": int,
-    "max_weight": convert_str_to_number,
-    "min_weight": convert_str_to_number,
-    "min_edge": int,
-    "max_edge": int,
-    "sign": convert_str_to_bool,
-    "direct": convert_str_to_bool,
-    "self_loop": convert_str_to_bool,
-    "multigraph": convert_str_to_bool,
-    "config": convert_str_to_bool,
+def handle_str_to_bool(string):
+    """
+    Convert 0/1 string to bool and raise ValueError if string is invalid.
+
+    :param string: input string
+    :type string: str
+    :return: result as bool
+    """
+    val = int(string)
+    if val not in [0, 1]:
+        raise ValueError
+    return bool(val)
+
+
+def handle_output_format(string):
+    """
+    Convert string to output format index.
+
+    :param string: input string
+    :type string: str
+    :return: output format index as int
+    """
+    output_format = handle_pos_int(string)
+    if output_format not in pyrgg.params.SUFFIX_MENU:
+        raise ValueError
+    return output_format
+
+
+def handle_engine(string):
+    """
+    Convert string to engine index.
+
+    :param string: input string
+    :type string: str
+    :return: engine index as int
+    """
+    engine = handle_pos_int(string)
+    if engine not in pyrgg.params.ENGINE_MENU:
+        raise ValueError
+    return engine
+
+
+ITEM_HANDLERS = {
+    "file_name": handle_string,
+    "output_format": handle_output_format,
+    "weight": handle_str_to_bool,
+    "engine": handle_engine,
+    "vertices": handle_pos_int,
+    "number_of_files": handle_pos_int,
+    "max_weight": handle_str_to_number,
+    "min_weight": handle_str_to_number,
+    "min_edge": handle_pos_int,
+    "max_edge": handle_pos_int,
+    "sign": handle_str_to_bool,
+    "direct": handle_str_to_bool,
+    "self_loop": handle_str_to_bool,
+    "multigraph": handle_str_to_bool,
+    "config": handle_str_to_bool,
+    "probability": handle_str_prob,
 }
 
 
@@ -111,7 +252,7 @@ def description_print():
     print(pyrgg.params.PYRGG_LINKS)
     line(40)
     print("\n")
-    print(fill(pyrgg.params.PYRGG_DESCRIPTION, width=100))
+    print(pyrgg.params.PYRGG_DESCRIPTION)
     print("\n")
     line(40)
 
@@ -135,7 +276,7 @@ def convert_bytes(num):
 
     :param num: the input number.
     :type num: int
-    :return: str
+    :return: result as str
     """
     for x in ['bytes', 'KB', 'MB', 'GB', 'TB']:
         if num < 1024.0:
@@ -156,70 +297,6 @@ def filesize(fileaddr):  # pragma: no cover
     print("Graph File Size : " + convert_bytes(file_size))
 
 
-def logger(
-        file_name,
-        vertices_number,
-        edge_number,
-        max_edge,
-        min_edge,
-        directed,
-        signed,
-        multigraph,
-        self_loop,
-        max_weight,
-        min_weight,
-        elapsed_time):
-    """
-    Save generated graphs log.
-
-    :param file_name: file name
-    :type file_name: str
-    :param vertices_number: number of vertices
-    :type vertices_number:int
-    :param edge_number: number of edges
-    :type edge_number: int
-    :param max_edge: maximum number of edges (connected to each vertex)
-    :type max_edge: int
-    :param min_edge: minimum number of edges (connected to each vertex)
-    :type min_edge: int
-    :param directed: directed
-    :type directed: int
-    :param signed: weight sign flag
-    :type signed: int
-    :param multigraph: multigraph flag
-    :type multigraph: int
-    :param self_loop: self loop flag
-    :type self_loop: int
-    :param max_weight: maximum weight
-    :type max_weight: int
-    :param min_weight: minimum weight
-    :type min_weight: int
-    :param elapsed_time: elapsed time
-    :type elapsed_time : str
-    :return:  None
-    """
-    try:
-        with open("logfile.log", "a") as file:
-            file.write(pyrgg.params.PYRGG_LOGGER_TEMPLATE.format(datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
-                                                                 file_name,
-                                                                 str(vertices_number),
-                                                                 str(edge_number),
-                                                                 str(max_edge),
-                                                                 str(min_edge),
-                                                                 str(bool(directed)),
-                                                                 str(bool(signed)),
-                                                                 str(bool(multigraph)),
-                                                                 str(bool(self_loop)),
-                                                                 str(is_weighted(max_weight,
-                                                                                 min_weight,
-                                                                                 bool(signed))),
-                                                                 str(max_weight),
-                                                                 str(min_weight),
-                                                                 elapsed_time))
-    except Exception:
-        print(pyrgg.params.PYRGG_LOGGER_ERROR_MESSAGE)
-
-
 def time_convert(input_time):
     """
     Convert input_time from sec to DD,HH,MM,SS format.
@@ -234,7 +311,7 @@ def time_convert(input_time):
     value_dict["d"], value_dict["s"] = divmod(value_dict["s"], 24 * 3600)
     value_dict["h"], value_dict["s"] = divmod(value_dict["s"], 3600)
     value_dict["m"], value_dict["s"] = divmod(value_dict["s"], 60)
-    for i in postfix_dict.keys():
+    for i in postfix_dict:
         if value_dict[i] != 1:
             postfix_dict[i] += "s"
     return ", ".join([
@@ -272,16 +349,10 @@ def input_filter(input_dict):
     if not filtered_dict["self_loop"]:
         edge_upper_threshold -= 1
 
-    if filtered_dict["output_format"] not in list(
-            range(1, len(pyrgg.params.SUFFIX_MENU) + 1)):
-        filtered_dict["output_format"] = 1
-
     if not filtered_dict["multigraph"]:
         for key in ["min_edge", "max_edge"]:
             filtered_dict[key] = min(filtered_dict[key], edge_upper_threshold)
 
-    filtered_dict["number_of_files"] = max(1, filtered_dict["number_of_files"])
-
     return filtered_dict
 
 
@@ -304,20 +375,23 @@ def get_input(input_func=input):
         "sign": True,
         "output_format": 1,
         "weight": True,
+        "engine": 1,
         "direct": True,
         "self_loop": True,
         "multigraph": False,
         "config": False,
+        "probability": 0.5,
     }
 
-    result_dict = _update_using_first_menu(result_dict, input_func)
-    result_dict = _update_using_second_menu(result_dict, input_func)
+    result_dict = _update_using_menu(result_dict, input_func)
+    result_dict = _update_with_engine_params(
+        result_dict, input_func, pyrgg.params.ENGINE_PARAM_MAP[result_dict['engine']])
     return input_filter(result_dict)
 
 
-def _update_using_first_menu(result_dict, input_func):
+def _update_using_menu(result_dict, input_func):
     """
-    Update result_dict using user input from the first menu.
+    Update result_dict using user input from the menu.
 
     :param result_dict: result data
     :type result_dict: dict
@@ -325,39 +399,11 @@ def _update_using_first_menu(result_dict, input_func):
     :type input_func: function object
     :return: result_dict as dict
     """
-    MENU_ITEMS_KEYS1 = sorted(list(pyrgg.params.MENU_ITEMS1.keys()))
-    for item in MENU_ITEMS_KEYS1:
+    for index in sorted(pyrgg.params.MENU_ITEMS):
+        item1, item2 = pyrgg.params.MENU_ITEMS[index]
         while True:
             try:
-                result_dict[item] = ITEM_CONVERTORS[item](
-                    input_func(pyrgg.params.MENU_ITEMS1[item])
-                )
-            except Exception:
-                print(pyrgg.params.PYRGG_INPUT_ERROR_MESSAGE)
-            else:
-                break
-    return result_dict
-
-
-def _update_using_second_menu(result_dict, input_func):
-    """
-    Update result_dict using user input from the second menu.
-
-    :param result_dict: result data
-    :type result_dict: dict
-    :param input_func: input function
-    :type input_func: function object
-    :return: result_dict as dict
-    """
-    MENU_ITEMS_KEYS2 = sorted(list(pyrgg.params.MENU_ITEMS2.keys()))
-    for index in MENU_ITEMS_KEYS2:
-        item1 = pyrgg.params.MENU_ITEMS2[index][0]
-        item2 = pyrgg.params.MENU_ITEMS2[index][1]
-        if not result_dict["weight"] and item1 in ["max_weight", "min_weight"]:
-            continue
-        while True:
-            try:
-                result_dict[item1] = ITEM_CONVERTORS[item1](
+                result_dict[item1] = ITEM_HANDLERS[item1](
                     input_func(item2)
                 )
             except Exception:
@@ -367,207 +413,32 @@ def _update_using_second_menu(result_dict, input_func):
     return result_dict
 
 
-def _threshold_calc(min_edge, max_edge, vertex_degree):
+def _update_with_engine_params(result_dict, input_func, engine_params):
     """
-    Calculate threshold for branch_gen function.
+    Update result_dict using user input based on given engine requirements.
 
-    :param min_edge: minimum number of edges (connected to each vertex)
-    :type min_edge: int
-    :param max_edge: maximum number of edges (connected to each vertex)
-    :type max_edge: int
-    :param vertex_degree: vertex degree
-    :type vertex_degree: int
-    :return: threshold as int
+    :param result_dict: result data
+    :type result_dict: dict
+    :param input_func: input function
+    :type input_func: function object
+    :param engine_params: engine parameters
+    :type engine_params: dict
+    :return: result_dict as dict
     """
-    threshold = min_edge
-    lower_limit = 0
-    upper_limit = max_edge - vertex_degree
-    if vertex_degree < min_edge:
-        lower_limit = min_edge - vertex_degree
-    if upper_limit > lower_limit:
-        threshold = randint(lower_limit, upper_limit)
-    return threshold
-
-
-def branch_gen(
-        vertex_index,
-        max_edge,
-        min_edge,
-        min_weight,
-        max_weight,
-        precision,
-        sign,
-        direct,
-        self_loop,
-        multigraph,
-        used_vertices,
-        degree_dict,
-        degree_sort_dict):
-    """
-    Generate branch and weight vector of each vertex.
-
-    :param vertex_index: origin vertex index
-    :type vertex_index: int
-    :param max_edge: maximum number of edges (connected to each vertex)
-    :type max_edge: int
-    :param min_edge: minimum number of edges (connected to each vertex)
-    :type min_edge: int
-    :param min_weight: weight min range
-    :type min_weight: int
-    :param max_weight: weight max range
-    :type max_weight: int
-    :param precision: numbers precision
-    :type precision: int
-    :param sign: weight sign flag
-    :type sign: bool
-    :param direct: directed and undirected graph flag
-    :type direct: bool
-    :param self_loop: self loop flag
-    :type self_loop: bool
-    :param multigraph: multigraph flag
-    :type multigraph: bool
-    :param used_vertices: used vertices dictionary
-    :type used_vertices: dict
-    :param degree_dict: all vertices degree
-    :type degree_dict: dict
-    :param degree_sort_dict: degree to vertices list
-    :type degree_sort_dict: dict
-    :return: branch and weight list
-    """
-    index = 0
-    branch_list = []
-    weight_list = []
-    reference_vertices = []
-    random_unit = randint
-    vertex_degree = degree_dict[vertex_index]
-    if vertex_degree >= max_edge:
-        return [branch_list, weight_list]
-    threshold = _threshold_calc(
-        min_edge=min_edge,
-        max_edge=max_edge,
-        vertex_degree=vertex_degree)
-    for i in range(max_edge + 1):
-        reference_vertices.extend(list(degree_sort_dict[i].values()))
-        if len(reference_vertices) >= threshold:
-            break
-    if precision > 0:
-        random_unit = uniform
-    if not direct and (
-            vertex_index in used_vertices) and not multigraph:
-        reference_vertices = list(
-            set(reference_vertices) - set(used_vertices[vertex_index]))
-    if not self_loop and vertex_index in reference_vertices:
-        reference_vertices.remove(vertex_index)
-    if pyrgg.params.PYRGG_TEST_MODE:
-        reference_vertices.sort()
-    while (index < threshold):
-        vertex_degree = degree_dict[vertex_index]
-        if vertex_degree >= max_edge:
-            break
-        if len(reference_vertices) == 0:
-            break
-        random_tail_index = choice(
-            range(len(reference_vertices)))
-        random_tail = reference_vertices[random_tail_index]
-        random_tail_degree = degree_dict[random_tail]
-        if random_tail_degree >= max_edge or (
-            random_tail == vertex_index and random_tail_degree >= (
-                max_edge - 1)):
-            reference_vertices.pop(random_tail_index)
+    for index in sorted(engine_params):
+        item1, item2 = engine_params[index]
+        if not result_dict["weight"] and item1 in ["max_weight", "min_weight"]:
             continue
-        if not direct:
+        while True:
             try:
-                used_vertices[random_tail].append(vertex_index)
-            except KeyError:
-                used_vertices[random_tail] = [vertex_index]
-        weight_sign = 1
-        if sign:
-            weight_sign = choice([1, -1])
-        random_weight = weight_sign * random_unit(min_weight, max_weight)
-        random_weight = round(random_weight, precision)
-        branch_list.append(random_tail)
-        weight_list.append(random_weight)
-        index += 1
-        del degree_sort_dict[vertex_degree][vertex_index]
-        degree_dict[random_tail] += 1
-        degree_dict[vertex_index] += 1
-        degree_sort_dict[degree_dict[vertex_index]
-                         ][vertex_index] = vertex_index
-        if random_tail != vertex_index:
-            del degree_sort_dict[random_tail_degree][random_tail]
-            degree_sort_dict[degree_dict[random_tail]
-                             ][random_tail] = random_tail
-        if not multigraph:
-            reference_vertices.pop(random_tail_index)
-    return [branch_list, weight_list]
-
-
-def edge_gen(
-        vertices_number,
-        min_weight,
-        max_weight,
-        min_edge,
-        max_edge,
-        sign,
-        direct,
-        self_loop,
-        multigraph):
-    """
-    Generate each vertex connection number.
-
-    :param vertices_number: number of vertices
-    :type vertices_number: int
-    :param min_weight: weight min range
-    :type min_weight: int
-    :param max_weight: weight max range
-    :type max_weight: int
-    :param min_edge: minimum number of edges (connected to each vertex)
-    :type min_edge: int
-    :param max_edge: maximum number of edges (connected to each vertex)
-    :type max_edge: int
-    :param sign: weight sign flag
-    :type sign: bool
-    :param direct: directed and undirected graph flag
-    :type direct: bool
-    :param self_loop: self loop flag
-    :type self_loop: bool
-    :param multigraph: multigraph flag
-    :type multigraph: bool
-    :return: list of dicts
-    """
-    precision = max(
-        get_precision(max_weight),
-        get_precision(min_weight))
-    temp = 0
-    vertices_id = list(range(1, vertices_number + 1))
-    vertices_edge = []
-    weight_list = []
-    used_vertices = {}
-    degree_sort_dict = {i: {} for i in range(max_edge + 1)}
-    degree_dict = {}
-    for i in vertices_id:
-        degree_dict[i] = 0
-        degree_sort_dict[0][i] = i
-    branch_gen_params = {
-        "max_edge": max_edge,
-        "min_edge": min_edge,
-        "min_weight": min_weight,
-        "max_weight": max_weight,
-        "sign": sign,
-        "direct": direct,
-        "self_loop": self_loop,
-        "multigraph": multigraph,
-        "used_vertices": used_vertices,
-        "degree_dict": degree_dict,
-        "degree_sort_dict": degree_sort_dict,
-        "precision": precision}
-    for i in vertices_id:
-        temp_list = branch_gen(vertex_index=i, **branch_gen_params)
-        vertices_edge.append(temp_list[0])
-        weight_list.append(temp_list[1])
-        temp = temp + len(temp_list[0])
-    return [dict(zip(vertices_id, vertices_edge)),
-            dict(zip(vertices_id, weight_list)), temp]
+                result_dict[item1] = ITEM_HANDLERS[item1](
+                    input_func(item2)
+                )
+            except Exception:
+                print(pyrgg.params.PYRGG_INPUT_ERROR_MESSAGE)
+            else:
+                break
+    return result_dict
 
 
 def json_to_yaml(filename):
@@ -610,14 +481,15 @@ def save_config(input_dict):
 
     :param input_dict: input data
     :type input_dict: dict
-    :return: path to file
+    :return: path to file as str
     """
     try:
         input_dict_temp = input_dict.copy()
-        input_dict_temp["engine"] = "pyrgg"
+        input_dict_temp['engine'] = pyrgg.params.ENGINE_MENU[input_dict_temp['engine']]
         input_dict_temp['pyrgg_version'] = pyrgg.params.PYRGG_VERSION
         input_dict_temp['output_format'] = pyrgg.params.OUTPUT_FORMAT[input_dict_temp['output_format']]
-        fname = pyrgg.params.CONFIG_FILE_FORMAT.format(input_dict_temp['file_name'])
+        fname = pyrgg.params.CONFIG_FILE_FORMAT.format(
+            input_dict_temp['file_name'])
         with open(fname, "w") as json_file:
             json_dump(input_dict_temp, json_file, indent=2)
         return os.path.abspath(fname)
@@ -631,12 +503,13 @@ def load_config(path):
 
     :param path: path to config file
     :type path: str
-    :return: input data
+    :return: input data as dict
     """
     try:
         with open(path, "r") as json_file:
             config = json_loads(json_file.read())
             config['output_format'] = pyrgg.params.OUTPUT_FORMAT_INV[config['output_format']]
+            config['engine'] = pyrgg.params.ENGINE_MENU_INV[config['engine']]
             return input_filter(config)
     except BaseException:
         print(pyrgg.params.PYRGG_CONFIG_LOAD_ERROR_MESSAGE)
@@ -650,15 +523,14 @@ def _print_select_config(configs, input_func=input):
     :type configs: list
     :param input_func: input function
     :type input_func: function object
-    :return: input data
+    :return: input data as dict
     """
     if len(configs) == 0:
         return None
-    print("Config files detected in current directory are listed below:")
+    print(pyrgg.params.PYRGG_CONFIG_LIST_MESSAGE)
     for i, config in enumerate(configs):
         print("[{}] - {}".format(i + 1, config))
-    key = input_func(
-        "Press the config index to load or any other keys to start with new one : ")
+    key = input_func(pyrgg.params.PYRGG_CONFIG_LOAD_MESSAGE)
     try:
         return load_config(configs[int(key) - 1])
     except BaseException:
@@ -671,7 +543,7 @@ def check_for_config(input_func=input):
 
     :param input_func: input function
     :type input_func: function object
-    :return: input data
+    :return: input data as dict
     """
     configs = []
     for filename in os.listdir(pyrgg.params.SOURCE_DIR):
